﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Globalization;
using ZedGraph;

namespace jordan_svd
{
    /**
     * The main class that includes the user interface and initialise the input values.
     **/
    public partial class ErgasiaTiftikoglou : Form
    {
        //We define the culture that we are going to use for the numbers
        protected CultureInfo ci = new CultureInfo("en-US");

        // The input array for the SVD
        public double[][] A;

        // The array list of arrays for polynomial 
        public ArrayList polionimikos;

        // The class that solves SVD
        protected qrbsvd jsvd;

        // The class that solves polynomial arrays
        protected ZeroPolynomialMatrixCalculation polionimAlgorithm;

        /**
         * The default constructor of the class
         **/
        public ErgasiaTiftikoglou()
        {
            // Initializes the user interface
            InitializeComponent();
            // Initialises the Graph
            iniZed();
        }

        /**
         * Initialises the Graph
         */
        protected void iniZed()
        {
            GraphPane myPane = zedG.GraphPane;

            myPane.CurveList.Clear();
            myPane.GraphObjList.Clear();

            myPane.Title.Text = "Χρόνος Εκτέλεσης ";
            myPane.XAxis.Title.Text = "Διάσταση πολυώνυμικων πινάκων";
            myPane.YAxis.Title.Text = "Χρόνος";

            zedG.Invalidate(true);
            zedG.Refresh();

        }

        /**
         * Trim the spaces from the d and returns the number that represents
         */
        public double getNormalized(string d)
        {
            d = d.Trim();
            return double.Parse(d, ci);
        }

        ////////////////////////////////////////////////////////////////////////////////////////
        // SVD 
        ////////////////////////////////////////////////////////////////////////////////////////
        /**
         * Parse the line and returns an array with numbers.
         * The numbers that returned are double precision
         * The numbers in the line must be separated with the space character
         */
        public double[] getDoubles(string line)
        {
            double[] result;
            char[] del = { ' ' };
            line = line.Trim();
            string[] parts;

            parts = line.Split(del);
            //result = new double[parts.Length + 1];
            result = new double[parts.Length];
            for (int i = 0; i < parts.Length; i++)
            {
                //result[i + 1] = getNormalized(parts[i]);
                result[i] = getNormalized(parts[i]);
            }
            return result;
        }


        /**
        * Parse the line and returns an array with numbers.
        * The numbers that returned are integers
        * The numbers in the line must be separated with the space character
        */
        public int[] getInts(string line)
        {
            int[] result;
            char[] del = { ' ' };
            line = line.Trim();
            string[] parts;

            parts = line.Split(del);
            result = new int[parts.Length];
            for (int i = 0; i < parts.Length; i++)
            {
                parts[i] = parts[i].Trim();
                result[i] = int.Parse(parts[i], ci);
            }
            return result;
        }


        /**
         * Reads the specified file and returns a two dimensional array.
         * Throws an exception if an error occurred
         */
        public double[,] readMatrixFromFile(string fileName)
        {
            int lastStiles;
            int lastGrammes;
            string line;
            int[] info;
            double[,] A;

            try
            {
                // Open the file
                TextReader arxeio = new StreamReader(fileName);
                // Read the first Line 
                line = arxeio.ReadLine();
                info = getInts(line);
                lastStiles = info[1];
                lastGrammes = info[0];

                A = new double[lastGrammes, lastStiles];
                double[] ll;
                for (int i = 0; i < lastGrammes; i++)
                {
                    line = arxeio.ReadLine();
                    ll = getDoubles(line);
                    for (int j = 0; j < lastStiles; j++)
                    {
                        A[i, j] = ll[j];
                    }
                }

                // close the file 
                arxeio.Close();

            }
            catch (Exception ex)
            {
                MessageBox.Show("Δεν είναι δυνατή η ανάγνωση του αρχείου <" +
                      fileName + ">", "Λάθος", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return null;
            }
            return A;
        }


        /**
         * Handler that called when the user press the "Ανάγνωση Πίνακα" button.
         * Reads the file that is specified in the "pinakasName" text field.
         * Performs the SVD algorithm to that array
         */
        private void readM_Click(object sender, EventArgs e)
        {
            double[,] p = readMatrixFromFile(pinakasName.Text);

            A = new double[p.Length][];
            int m = p.GetLength(0);
            int n = p.GetLength(1);
            for (int i = 0; i < m; i++)
            {
                A[i] = new double[n];
            }
            for (int i = 0; i < m; i++)
            {
                for (int j = 0; j < n; j++)
                {
                    A[i][j] = p[i, j];
                }
            }

            jsvd = new qrbsvd();
            jsvd.solveSvd(p);
            renderSVD();
        }

        /**
         * Handler that called when the user press the "Ορισμός τιμών Α" button.
         * User inserts the A array.
         * Performs the SVD algorithm to that array
         */
        private void orismos_Click(object sender, EventArgs e)
        {
            Times s = new Times();
            s.ShowDialog(this);
            String[] l;

            if (s.DialogResult == DialogResult.OK)
            {

                string[] dlines = s.arrayATxt.Lines;
                ArrayList tmpL = new ArrayList();
                for (int i = 0; i < dlines.Length; i++)
                {
                    dlines[i] = dlines[i].Trim();
                    if (dlines[i].Length != 0)
                    {
                        tmpL.Add(dlines[i]);
                    }
                }


                int lines = tmpL.Count;
                l = ((String)tmpL[0]).Split(' ');
                int cols = l.Length;
                double[,] p;


                p = new double[lines, cols];
                A = new double[lines][];

                for (int i = 0; i < lines; i++)
                {
                    A[i] = new double[cols];
                    l = ((String)tmpL[i]).Split(' ');

                    for (int j = 0; j < cols; j++)
                    {
                        p[i, j] = Double.Parse(l[j], ci);
                        A[i][j] = p[i, j];
                    }
                }

                // svd4
                jsvd = new qrbsvd();
                //jsvd.solveAlgSvd(p);
                jsvd.solveSvd(p);
                renderSVD();
            }
        }


        /**
         * Renders the results of the SVD
         */
        private void renderSVD()
        {

            MArrayTxt.Text = "";
            MArray2Txt.Text = "";
            UArrayTxt.Text = "";
            UArray2Txt.Text = "";
            SArrayTxt.Text = "";
            SArray2Txt.Text = "";
            VArrayTxt.Text = "";
            VTArrayTxt.Text = "";
            VTArray2Txt.Text = "";

            for (int i = 0; i < jsvd.rowDimension; i++)
            {
                string line = "";
                for (int j = 0; j < jsvd.columnDimention; j++)
                {
                    line = line + String.Format("{0,9:0.0000}", A[i][j]) + " ";

                }
                MArrayTxt.AppendText(line + "\n");
                MArray2Txt.AppendText(line + "\n");
            }

            int gr = jsvd.U.GetLength(0);
            int gc = jsvd.U.GetLength(1);
            for (int i = 0; i < gr; i++)
            {
                string line = "";
                for (int j = 0; j < gc; j++)
                {
                    line = line + String.Format("{0,9:0.00000}", jsvd.U[i, j]) + " ";

                }
                UArrayTxt.AppendText(line + "\n");
                UArray2Txt.AppendText(line + "\n");
            }

            gr = jsvd.S.GetLength(0);
            gc = jsvd.S.GetLength(1);
            for (int i = 0; i < gr; i++)
            {
                string line = "";
                for (int j = 0; j < gc; j++)
                {
                    line = line + String.Format("{0,9:0.00000}", jsvd.S[i, j]) + " ";

                }
                SArrayTxt.AppendText(line + "\n");
                SArray2Txt.AppendText(line + "\n");
            }

            gr = jsvd.V.GetLength(0);
            gc = jsvd.V.GetLength(1);
            for (int i = 0; i < gr; i++)
            {
                string line = "";
                for (int j = 0; j < gc; j++)
                {
                    line = line + String.Format("{0,9:0.00000}", jsvd.V[i, j]) + " ";

                }
                VArrayTxt.AppendText(line + "\n");
            }

            double[,] zz = jsvd.transpose(jsvd.V);
            gr = zz.GetLength(0);
            gc = zz.GetLength(1);
            for (int i = 0; i < gr; i++)
            {
                string line = "";
                for (int j = 0; j < gc; j++)
                {
                    line = line + String.Format("{0,9:0.00000}", zz[i, j]) + " ";
                }
                VTArrayTxt.AppendText(line + "\n");
                VTArray2Txt.AppendText(line + "\n");
            }
        }


        ////////////////////////////////////////////////////////////////////////////////////////
        // Zero polynomial matrix calculation
        ////////////////////////////////////////////////////////////////////////////////////////

        /**
         * Reads the polynomial arrays from user and calculates the result
         */
        private void polyButton_Click(object sender, EventArgs e)
        {
            Times s = new Times();
            s.ShowDialog(this);

            if (s.DialogResult == DialogResult.OK)
            {
                polyCalc(s.arrayATxt.Lines);
            }
        }

        /**
         * Parse the string lines into arrays of double 
         * Performs the calculations of zero polynomial matrix 
         */
        private void polyCalc(string[] dlines)
        {
            String[] l;
            ArrayList mainL = new ArrayList();
            ArrayList tmpL = new ArrayList(); ;


            // we read the lines as string arrays
            for (int i = 0; i < dlines.Length; i++)
            {
                dlines[i] = dlines[i].Trim();
                if (dlines[i].Length != 0)
                {
                    tmpL.Add(dlines[i]);
                }
                else
                {
                    if (tmpL.Count != 0)
                    {
                        mainL.Add(tmpL);
                    }
                    tmpL = new ArrayList();
                }
            }


            int arrays = mainL.Count;
            polionimikos = new ArrayList();
            // We parse the string arrays and we create the arrays of reals
            for (int k = 0; k < arrays; k++)
            {
                tmpL = (ArrayList)mainL[k];

                int lines = tmpL.Count;
                l = ((String)tmpL[0]).Split(' ');
                int cols = l.Length;
                double[,] p;

                p = new double[lines, cols];

                for (int i = 0; i < lines; i++)
                {
                    l = ((String)tmpL[i]).Split(' ');
                    for (int j = 0; j < cols; j++)
                    {
                        p[i, j] = Double.Parse(l[j], ci);
                    }
                }
                polionimikos.Add(p);
            }

            // we solve
            polionimAlgorithm = new ZeroPolynomialMatrixCalculation();
            polionimAlgorithm.setPolionimikoPinaka(polionimikos);
            polionimAlgorithm.solve();

            bool conRes = polionimAlgorithm.condtest(polionimikos);
            int[] polaplotita = polionimAlgorithm.calculatePolaplotita();
            // Renders the result to screen
            renderPolionimResults(conRes , polaplotita);
        }

        /**
         * Outputs the results in the screen
         */
        private void renderPolionimResults(bool con , int []polaplotia)
        {
            poliResultText.AppendText("==========================================");
            poliResultText.AppendText("\n");
            poliResultText.AppendText("==========================================");
            poliResultText.AppendText("\n");


            int dinami = polionimikos.Count - 1;
            string spaces = "           ";
            for (int i = 0; i < polionimikos.Count; i++)
            {
                double[,] tmp = (double[,])polionimikos[i];
                int grames = tmp.GetLength(0);
                int stiles = tmp.GetLength(1);
                int posC = grames / 2;

                for (int j = 0; j < grames; j++)
                {
                    poliResultText.AppendText("\n");
                    if (j == posC)
                    {
                        poliResultText.AppendText("C( " + String.Format("{0,3:0}", (dinami - i)) + " ) = ");
                    }
                    else
                    {
                        poliResultText.AppendText(spaces);
                    }

                    for (int k = 0; k < stiles; k++)
                    {
                        String num = String.Format("{0,5:0.00}", tmp[j, k]);
                        poliResultText.AppendText(" " + num + " ");

                    }

                }
                poliResultText.AppendText("\n");

            }
            int pres = polionimAlgorithm.resultReal.Length;
            poliResultText.AppendText("\n");
            poliResultText.AppendText("\n");
            poliResultText.AppendText("\n");

            poliResultText.AppendText("                  ΛΥΣΗ");
            poliResultText.AppendText("\n");
            poliResultText.AppendText("---------------------------------------");
            poliResultText.AppendText("\n");

            for (int i = 0; i < pres; i++)
            {
                poliResultText.AppendText(" " + String.Format("{0,9:0.00000}", polionimAlgorithm.resultReal[i]) +
                    "    " + String.Format("{0,9:0.00000}", polionimAlgorithm.resultIm[i]) + "     " + polaplotia[i]);

                poliResultText.AppendText("\n");
            }

            poliResultText.AppendText("\n");
            if ( con ) {
                poliResultText.AppendText("Οι πολυωνυμικοί πίνακες έχουν τα ίδια μηδενικά");
                
            }else {
                poliResultText.AppendText("Οι πολυωνυμικοί πίνακες δεν έχουν τα ίδια μηδενικά");
            }
            poliResultText.AppendText("\n");

            long ticksPerSecond;
            PerformanceCounter ct = new PerformanceCounter();
            ticksPerSecond = ct.QueryPerformanceFrequency();
            poliResultText.AppendText("\n");
            poliResultText.AppendText("Ο αλγόριθμος λύσης SVD καλέστηκε " + polionimAlgorithm.svdCount
                + " φορές και εκτελέστηκε σε " + (int)polionimAlgorithm.svdTotalTime/1000 + " χιλιοστά του δευτερολέπτου");
            poliResultText.AppendText("\n");

            poliResultText.AppendText("Για τον υπολογισμό των ιδιοτιμών χρειάστηκαν " +(int)polionimAlgorithm.eigTotalTime / 1000 + " χιλιοστά του δευτερολέπτου.");
            poliResultText.AppendText("\n");

            poliResultText.AppendText("==========================================");
            poliResultText.AppendText("\n");
            poliResultText.AppendText("==========================================");
            poliResultText.AppendText("\n");
        }

        /**
         * Reads the file that contains the  polynomial arrays.
         */
        private void readFilePolBtn_Click(object sender, EventArgs e)
        {
            String all = null;
            try
            {
                // Open the file
                TextReader arxeio = new StreamReader(polFileName.Text);
                all = arxeio.ReadToEnd();
                arxeio.Close();
            }
            catch (Exception ex)
            {
                MessageBox.Show("Δεν είναι δυνατή η ανάγνωση του αρχείου <" +
                      polFileName.Text + ">", "Λάθος", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            String[] split = { "\r\n" };
            if (all != null)
            {
                String[] dlines = all.Split(split, System.StringSplitOptions.None);
                polyCalc(dlines);

                
            }
        }

        /**
         * Cleans the panel that contains the results of  polynomial arrays
         */
        private void cleanBtn_Click(object sender, EventArgs e)
        {
            poliResultText.Text = "";
        }



        ////////////////////////////////////////////////////////////////////////////////////////
        // Zero polynomial matrix calculation Speed Calculation
        ////////////////////////////////////////////////////////////////////////////////////////
        /**
         * Calculates the speed of zero polynomial matrix calculation.
         * Outputs the graph as result.
         */
        private void calcXronous_Click(object sender, EventArgs e)
        {
            try
            {
                int v, s;

                v = int.Parse(pvathmos.Text, ci);
                s = int.Parse(psize.Text, ci);

                PoliSpeedAnalisys sp = new PoliSpeedAnalisys();
                sp.performCalculations(s, v, this.calcInfo);
                outputGraph(sp.times);
            }
            catch (Exception ex)
            {
                MessageBox.Show("Σφάλμα:" + ex.Message, "Λάθος", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

        }

        /**
         * Draws the  speed graph of zero polynomial matrix calculation.
         */
        private void outputGraph(long[] itimes)
        {
            GraphPane myPane = zedG.GraphPane;

            long[] times = new long[itimes.Length - 3];
            for (int i = 0; i < times.Length; i++)
            {
                times[i] = itimes[i + 3];
            }
            myPane.CurveList.Clear();
            myPane.GraphObjList.Clear();
            myPane.Title.Text = "Χρόνος Εκτέλεσης ";
            myPane.XAxis.Title.Text = "Διάσταση πολυώνυμικων πινάκων";
            myPane.YAxis.Title.Text = "Χρόνος";

            zedG.Invalidate(true);
            zedG.Refresh();

            // Build a PointPairList with points based on Sine wave
            PointPairList list = new PointPairList();
            for (int i = 0; i < times.Length; i++)
            {
                list.Add(i, times[i]);
            }

            // Hide the legend
            myPane.Legend.IsVisible = false;

            // Add a curve
            LineItem curve = myPane.AddCurve("label", list, Color.Red, SymbolType.Circle);
            curve.Line.Width = 2.0F;
            curve.Line.IsAntiAlias = true;
            curve.Symbol.Fill = new Fill(Color.White);
            curve.Symbol.Size = 7;

            // Fill the axis background with a gradient
            myPane.Chart.Fill = new Fill(Color.White, Color.FromArgb(255, Color.Bisque), 45.0F);

            // Offset Y space between point and label
            // NOTE:  This offset is in Y scale units, so it depends on your actual data
            const double offset = 1.0;

            int step = 1;
            if (times.Length > 30)
            {
                step = times.Length / 15;
            }

            // Loop to add text labels to the points
            for (int i = 0; i < times.Length; i += step)
            {
                // Get the pointpair
                PointPair pt = curve.Points[i];
                TextObj text = null;
                {
                    text = new TextObj("" + times[i], pt.X, pt.Y + offset,
                        CoordType.AxisXYScale, AlignH.Left, AlignV.Center);
                }

                text.ZOrder = ZOrder.A_InFront;
                // Hide the border and the fill
                text.FontSpec.Border.IsVisible = false;
                text.FontSpec.Fill.IsVisible = false;
                text.FontSpec.Angle = 90;
                myPane.GraphObjList.Add(text);
            }

            // Leave some extra space on top for the labels to fit within the chart rect
            myPane.YAxis.Scale.MaxGrace = 0.2;

            // Calculate the Axis Scale Ranges
            zedG.AxisChange();
            zedG.Invalidate(true);
        }

        private void ErgasiaTiftikoglou_Load(object sender, EventArgs e)
        {
            helpFile.LoadFile("odigies.rtf");
        }
    }
}
