﻿using System;
using System.Collections.Generic;
using System.Text;

namespace jordan_svd
{
    /**
     * Helper class for matrix.
     */ 
    public class GMatrix
    {

        /**
         * Create a clone from the a array
         */ 
        public static double[,] GetClone(double [,]a) {
            int m = a.GetLength(0);
            int n = a.GetLength(1);
            double[,] result = new double[m, n];
            for (int i = 0; i < m; i++)
            {
                for (int j = 0; j < n; j++)
                {
                    result[i, j] = a[i, j];
                }
            }
            return result;
        }


        /**
         * Linear algebraic matrix multiplication, A * B.
         * 
         */
        public static double[,] Multiply(double[,] a, double[,] b)
        {
            int m = a.GetLength(0);
            int n = b.GetLength(1);
            int k = a.GetLength(1);
            double[,] c = new double[m, n];
            alglib.rmatrixgemm(m, n, k, 1, a, 0, 0, 0, b, 0, 0, 0, 0, ref c, 0, 0);
            return c;
        }

        public static double[,] Pow(double[,] a , int cnt )
        {
            double[,] result;
            result = GetClone(a);
            for (int i = 1; i < cnt ; i++)
            {
                result = Multiply(result, a);
            }

            return result;
        }

        /**
         * Creates and returns an mXn array with zeroes
         */
        public static double[,] Zeroes(int m, int n)
        {
            double[,] result = new double[m,n];
            for (int i = 0; i < m; i++)
            {
                for (int j = 0; j < n; j++)
                {
                    result[i, j] = 0;
                }
            }
            return result;
        }


        /**
         * Creates and returns an mXn array with zeroes
        */
        public static double[] ZeroesOne( int n)
        {
            double[] result = new double[n];
            for (int i = 0; i < n; i++)
            {
                 result[i] = 0;
            }
            return result;
        }


        /**
         * Creates and returns an identity matrix m x n
         */
        public static double[,] Identity(int m, int n)
        {
            double[,] result = new double[m, n];
            for (int i = 0; i < m; i++)
            {
                for (int j = 0; j < n; j++)
                {
                    result[i, j] = (i == j ? 1.0 : 0.0);
                }
            }
            return result;
        }

        /**
         * Copies X into A. Copy starts at i0, j0 of A. 
         */ 
        public static void SetMatrix0(int i0, int i1, int j0, int j1, double[,]A , double[,] X)
        {
            try
            {
                for (int i = i0; i < i1; i++)
                {
                    for (int j = j0; j < j1; j++)
                    {
                        A[i, j] = X[i - i0, j - j0];
                    }
                }
            }
            catch (System.IndexOutOfRangeException e)
            {
                throw new System.IndexOutOfRangeException("Submatrix indices", e);
            }
        }

        /**
         * Copies the X into A size of A starting at r row and g column in X
         */ 
        public static void SetMatrix(int r, int g, double[,] A, double[,] X)
        {
            try
            {
                int gi = 0;
                int xRow = X.GetLength(0);
                int xCol = X.GetLength(1);
                for (int i = r; i < xRow + r; i++)
                {
                    int si = 0;
                    for (int j = g; j < g + xCol; j++)
                    {
                        A[i,j] = X[gi, si];
                        si++;
                    }
                    gi++;
                }
            }
            catch (System.IndexOutOfRangeException e)
            {
                throw new System.IndexOutOfRangeException("Submatrix indices", e);
            }
        }

        /**
         * Returns the transpose of the matrix inm
         */ 
        public static double[,] Transpose(double[,] inm)
        {
            int n = inm.GetLength(1);
            int m = inm.GetLength(0);
            double[,] result = new double[n, m];

            for (int i = 0; i < m; i++)
            {
                for (int j = 0; j < n; j++)
                {
                    result[j, i] = inm[i, j];
                }
            }
            return result;
        }

        /**
         * Returns the sub matrix of A.
         */ 
        public static double[,] GetMatrix2(int i0, int i1, int j0, int j1 , double[,]A)
        {
            double [,] X = new double[i1 - i0, j1 - j0];
            try
            {
                for (int i = i0; i < i1; i++)
                {
                    for (int j = j0; j < j1; j++)
                    {
                        X[i - i0 , j - j0] = A[i , j];
                    }
                }
            }
            catch (System.IndexOutOfRangeException e)
            {
                throw new System.IndexOutOfRangeException("Submatrix indices", e);
            }
            return X;
        }

        /**
         * Returns the sub matrix of A.
         */
        public static double[,] MulArrayWithNum(double num, double[,] A)
        {
            int g, s;
            g = A.GetLength(0);
            s = A.GetLength(1);
            double[,] result = new double[g,s];

            for (int i = 0; i < g; i++)
            {
                for (int j = 0; j < s; j++)
                {
                    result[i, j] = num * A[i, j];
                }
            }

            return result;
        }


        /**
         * Returns the trace of the matrix 
         */
        public static double Trace(double[,] A)
        {

            int n = A.GetLength(1);
            int m = A.GetLength(0);
            double result = 0;

            for (int i = 0; i < m; i++)
            {
                for (int j = 0; j < n; j++)
                {
                    if (i == j)
                    {
                        result = result + A[i, j];
                    }
                }
            }
            return result;
        }



    }
}
