function [ U, S, V ] = qrbsvd (A, tol)

if ( nargin == 1 )
   tol = 1024 * eps;
end

mdim = size (A);
maxiter = 100 * max (mdim);
count = 0;

% Initialize U, S, and V
U = eye (mdim(1));
S = A';
V = eye (mdim(2));

err = realmax;
while ( (err > tol) && (count < maxiter) )
    [Q, S] = qr (S'); 
    U = U * Q;
 

    [Q, S] = qr (S');
    V = V * Q;
    e = triu (S, 1);
    E = norm (e(:));
    F = norm (diag (S));

    if ( F == 0 )
      F = 1.0;
    end

    err = E / F;
    count = count + 1;
end

D = diag (S);
S = zeros (mdim);
for n = 1:length (D)
    Dn = D(n);
    S(n, n) = abs (Dn);
    if Dn < 0
       U(:, n) = - U(:, n);
    end
end

if (nargout <= 1)
   U = diag (S);
end
